;;;; sparse-vectors.debug.scm

(module (sparse-vectors debug)

(;export
  sparse-vector-value-kind
  sparse-vector-tree-fold
  sparse-vector-info
  sparse-vector->list)

(import
  scheme
  (chicken base)
  (chicken type)
  (chicken bitwise)
  (chicken fixnum)
  record-variants
  sparse-vectors
  (sparse-vectors hof))

(include-relative "sparse-vectors.types")

(: sparse-vector-value-kind (* --> symbol))
(: sparse-vector-tree-fold (sparse-vector ('a sparse-vector-node fixnum (or false sparse-vector-node) (or false fixnum) -> 'a) 'a --> 'a))
(: sparse-vector-info (sparse-vector --> fixnum fixnum fixnum))
(: sparse-vector->list (sparse-vector --> list))

;;

(include-relative "sparse-vectors-inlines")

(define (hilbert-node-fold node siz fun acc)
  (let loop ((i 0) (acc acc))
    (if (fx= i siz) acc
        (loop (fx+ i 1) (fun acc i (node-ref node i))) ) ) )

(define (hilbert-node-count hilbert)
  #; ;for testing
  (assert (= (hilbert-capacity hilbert)
             (sparse-vector-tree-fold hilbert (lambda (cnt nod hgt par idx) (fx+ cnt 1)) 0)))
  (hilbert-capacity hilbert) )

;;

(define (sparse-vector-value-kind val)
  (cond ((node? val) 'node)
        ((hilbert-def? val) 'unset!)
        (else 'set!)) )

(define (sparse-vector-tree-fold hilbert fun acc)
  (let ((siz (hilbert-node-size hilbert)))
    (let recur ((node (hilbert-root hilbert)) (height (hilbert-height hilbert)) (acc acc) (parent #f) (index #f))
      (if (not (node? node)) acc
          (let ((height-1 (fx- height 1))
                (acc (fun acc node height parent index)) )
            (if (fx= 0 height-1) acc
                (hilbert-node-fold
                  node siz
                  (lambda (acc i val) (recur val height-1 acc node i))
                  acc) ) ) ) ) ) )

(define (sparse-vector-info hilbert)
  (check-sparse-vector 'sparse-vector-info hilbert)
  (values
    (hilbert-node-count hilbert)
    (hilbert-height hilbert)
    (hilbert-node-size hilbert)) )

(define (sparse-vector->list hilbert)
  (sparse-vector-fold hilbert (lambda (a i v) (cons v a)) '() #f) )

) ;module (sparse-vectors debug)
